#ifndef _XRDOSSCACHE_API_H
#define _XRDOSSCACHE_API_H
/******************************************************************************/
/*                                                                            */
/*                X r d O s s C a c h e P l u g i n . h h                     */
/*                                                                            */
/* (c) 2007 by the Board of Trustees of the Leland Stanford, Jr., University  */
/*                            All Rights Reserved                             */
/*   Produced by Andrew Hanushevsky for Stanford University under contract    */
/*              DE-AC03-76-SFO0515 with the Department of Energy              */
/******************************************************************************/

//         $Id: XrdOssCachePlugin.hh,v 1.0 2009-10-19 00:00:50 ajp Exp $

#include <sys/types.h>
#include <errno.h>
#include "XrdSys/XrdSysHeaders.hh"

#include "XrdOss/XrdOss.hh"
#include "XrdOss/XrdOssApi.hh"
#include "XrdOssCache/XrdOssReadCache.hh"
#include "XrdOssCache/XrdOssCacheTrace.hh"
#include "XrdClient/XrdClient.hh"
#include "XrdClient/XrdClientAdmin.hh"
#include "XrdClient/XrdClientEnv.hh"

extern XrdOssSys* XrdOssSS;
extern XrdSysError OssEroute;

/******************************************************************************/
/*                            X r d O s s C a c h e F i l e                   */
/******************************************************************************/

class XrdSfsAio;
  
class XrdOssCacheFile : public XrdOssFile
{
public:
        XrdSysSemWait OpenSem; 
// The following two are virtual functions to allow for upcasting derivations
// of this implementation
//
virtual int     Close(long long *retsz=0);
virtual int     Open(const char *, int, mode_t, XrdOucEnv &);

virtual int     Fstat(struct stat *buf);
int     Fsync();
int     Fsync(XrdSfsAio *aiop);
int     Ftruncate(unsigned long long);
off_t   getMmap(void **addr);
  //int     isCompressed(char *cxidp=0);
ssize_t Read(               off_t, size_t);
ssize_t Read(       void *, off_t, size_t);
ssize_t ReadNoCache(       void *, off_t, size_t);
int     Read(XrdSfsAio *aiop);
ssize_t ReadRaw(    void *, off_t, size_t);
ssize_t Write(const void *, off_t, size_t);
int     Write(XrdSfsAio *aiop);

         // Constructor and destructor
         XrdOssCacheFile(const char *tid) : XrdOssFile (tid) {fd = -1; tident = tid; ch = 0; Mapped=false; MapUrl=""; Client=NULL; ClientAdmin=NULL;;BounceInterval = 10.0;}
         
virtual ~XrdOssCacheFile() {if (fd >= 0) Close();if (Client) delete Client; if (ClientAdmin) delete ClientAdmin;}
        XrdSysMutex ReadMutex;

private:

  const char *tident;
  const char *crPath;
  int   crOpts;
  
  bool  Maps2Local();
  bool  Maps2Remote();
  const char* GetMapping(const char* path, const char* secuser=NULL, bool iswrite=false);
  const char* GetMappingKey(const char* path, const char* key);
  bool  Mapped;
  XrdOucString MapUrl;
  XrdClient* Client;
  XrdClientAdmin* ClientAdmin;

  XrdOssReadCacheHandle* ch;
  float BounceInterval;
};

/******************************************************************************/
/*                             X r d O s s C a c h e S y s                    */
/******************************************************************************/
  
class XrdOucEnv;
class XrdSysError;
class XrdOucStream;
class XrdOucTList;

class XrdOssCacheSys : public XrdOssSys
{
public:
virtual XrdOssDF *newDir(const char *tident)
                       {return (XrdOssDF *)new XrdOssDir(tident);}
virtual XrdOssDF *newFile(const char *tident)
                       {return (XrdOssDF *)new XrdOssCacheFile(tident);}

bool      CreateLink(XrdOssReadCacheHandle* handle, const char* path);
int       Chmod(const char *, mode_t mode);
virtual
int       Create(const char *, const char *, mode_t, XrdOucEnv &, int opts=0);
int       Init(XrdSysLogger *, const char *);
int       Mkdir(const char *a, mode_t mode, int mkpath=0) { return XrdOssSys::Mkdir(a,mode,mkpath);}
int       Remdir(const char *a, int Opts=0) { return XrdOssSys::Remdir(a,Opts);}
int       Rename(const char *, const char *);
int       Stat(const char *, struct stat *, int resonly=0);
int       Truncate(const char *, unsigned long long);
int       Unlink(const char *, int Opts=0);

static char        *ConfigFN;       // -> Pointer to the config file name
static char        *myHost;
static char        *myName;
static long         PageSize;
static long         Size;
static long         Objects;   
static float        Threshold;
static char        *Directory;
static long         SyncInterval;
static long long    MapSize;
static bool         Public;
static bool         WriteSync;
static bool         NamespaceLink;
static XrdOucHash<XrdOucEnv> ProxyMap;
       XrdSysMutex  HandleMutex;
       XrdSysMutex  ProxyMapMutex;
       XrdSysMutex  ClientMutex;
  
static XrdOucHash<XrdOucString> *delaystore;

static int GetDelayValue(const char* tag);

         XrdOssCacheSys() {}
virtual ~XrdOssCacheSys() {}

XrdOssReadCache*    GetCache() { return Cache;}

private:

int    Configure(const char *);
int    ConfigProc(const char *ConfigFN);
int    ConfigXeq(char*, XrdOucStream&);
int    xtrac(XrdSysError *Eroute, XrdOucStream &Config, const char* var);
int    xcache(XrdSysError *Eroute, XrdOucStream &Config, const char* var);

bool   Maps2Local(const char* path);
bool   Maps2Remote(const char* path);
const char* GetMapping(XrdOucString &MapUrl, const char* path, const char* secuser=NULL, bool iswrite=false);

XrdOssReadCache* Cache; 

};

#endif
